#!BPY

"""
Name: 'BGC'
Blender: 236
Group: 'Wizards'
Tooltip: 'Make Camera spin and render 6 cubic images!'
"""

#############################################################
#                                                           #
# Blender Go Cubic                !                         #
#                                                           #
# (C) February 2005 Stefano <S68> Selleri                   #
# Released under the Blender Artistic Licence (BAL)         #
# See www.blender.org                                       #
#                                                           #
#############################################################
# History                                                   #
# V: 0.0.0 - 13-11-04 - Macouno published his clever tut    #
#                       on Panoramas, but I'm too lazy to   #
#                       set cameras by hand!                #
#    0.0.2 - 13-11-04 - A GUI!                              #
#############################################################

import Blender
from Blender import *

#############################################################
# GLOBALS                                                   #
#############################################################
VERSION         ='0.0.2'

EVENTHASH = {
    'NOEVENT': 1,
    'REDRAW': 2,
    'EXIT': 3,
    'DO': 4
    }

DATAHASH = {
    'SIZE': Draw.Create(100),
    'NAME': Draw.Create('/tmp/cube'),
    'TYPE': Draw.Create(1)
    }

#############################################################
# MAIN PROGRAM                                              #
#############################################################

def RenderIt():
    global DATA

    pi = 3.1415926535897932384626433832795

    cur = Scene.getCurrent()

    cntx = cur.getRenderingContext()
    cntx.imageSizeX(DATAHASH['SIZE'].val)
    cntx.imageSizeY(DATAHASH['SIZE'].val)
    cntx.aspectRatioX(1)
    cntx.aspectRatioY(1)
    cntx.setRenderWinSize(100)
    
    cntx.setRenderPath(DATAHASH['NAME'].val)
    if (DATAHASH['TYPE']==0):
        cntx.setImageType(Scene.Render.JPG)
    elif (DATAHASH['TYPE']==1):
        cntx.setImageType(Scene.Render.PNG)
    else:
        cntx.setImageType(Scene.Render.TARGA)
                              

    CamO = cur.getCurrentCamera()
    CamC = CamO.getData()

    CamC.setLens(16.0)

    ## First shot
    CamO.setEuler(pi/2,0.,0.)
    cntx.startFrame(1)
    cntx.endFrame(1)
    cntx.renderAnim()

    ## Second shot
    CamO.setEuler(pi/2,0.,-pi/2)
    cntx.startFrame(2)
    cntx.endFrame(2)
    cntx.renderAnim()

    ## Third shot
    CamO.setEuler(pi/2,0.,-pi)
    cntx.startFrame(3)
    cntx.endFrame(3)
    cntx.renderAnim()

    ## Fourth shot
    CamO.setEuler(pi/2,0.,-3*pi/2)
    cntx.startFrame(4)
    cntx.endFrame(4)
    cntx.renderAnim()

    ## Fifth shot
    CamO.setEuler(pi,0.,0.)
    cntx.startFrame(5)
    cntx.endFrame(5)
    cntx.renderAnim()

    ## Sixth shot
    CamO.setEuler(0.,0.,0.)
    cntx.startFrame(6)
    cntx.endFrame(6)
    cntx.renderAnim()

#############################################################
# MAIN WINDOW                                               #
#############################################################
def MainWindow():        
    global EVENTHASH,VERSION
    global DATA
    
    #############################################################
    # Backgrounds                                               #
    #############################################################

    BGL.glClearColor(0.5, 0.5, 0.5, 0.0)
    BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
    BGL.glColor3f(0, 0, 0) 			# Black
    BGL.glRectf(2, 2, 482, 520)
    BGL.glColor3f(0.48, 0.4, 0.57) 		# Light Purple
    BGL.glRectf(4, 479, 480, 510)
    BGL.glRectf(4, 34, 480, 450)
    BGL.glColor3f(0.3, 0.27, 0.35) 		# Dark purple
    BGL.glRectf(4, 451,480, 478)
    BGL.glRectf(4, 4, 480, 33)

    #############################################################
    # Common Header                                             #
    #############################################################

    BGL.glColor3f(0.9, 0.8, 1)
    BGL.glRasterPos2d(10, 500)
    Draw.Text("BGC - Blender Go Cube v. "+VERSION)
    BGL.glRasterPos2d(10, 484)
    Draw.Text("(C) Feb 2005 Stefano Selleri <a.k.a. S68>")
    Draw.Button("Exit", EVENTHASH['EXIT'], 407, 482, 70, 24)
         
    Draw.Button("GENERATE", EVENTHASH['DO'], 7, 7, 470, 24)
    BGL.glColor3f(0.9, 0.9, 0.9)

    #############################################################
    # Screen                                                    #
    #############################################################
    BGL.glRasterPos2d(10, 425)
    Draw.Text("Rendering Parameters:")

    DATAHASH['SIZE'] =  Draw.Slider(
                        "Image size: ", EVENTHASH['NOEVENT'], 20, 400, 210, 18,
                        DATAHASH['SIZE'].val, 4, 10000, 0,
                        "Images must be square...")

    DATAHASH['NAME'] =  Draw.String(
                        "Image prefix: ", EVENTHASH['NOEVENT'], 20, 380, 210, 18,
                        DATAHASH['NAME'].val, 128,
                        "Images will be named prefix0001.type ...")

    types = "Image type %t|JPG %x0|PNG %x1|Targa %x2"

    DATAHASH['TYPE'] = Draw.Menu(types, EVENTHASH['NOEVENT'],
                                 20, 360, 210, 18,
                                 DATAHASH['TYPE'].val,
                                 "Images will be this type")


#############################################################
# Graphics                                                  #
#############################################################
def draw():
    MainWindow()
    
#############################################################
# Event Handler                                             #
#############################################################
def event(evt, val): 
	if (evt== Draw.QKEY and not val):
		Draw.Exit()

def bevent(evt):
    global EVENTHASH

    if   (evt == EVENTHASH['EXIT']):
        print 'EXIT'
        Draw.Exit()
    elif (evt == EVENTHASH['DO']):
        RenderIt()
        Draw.Redraw()
	
#############################################################
# Registering all functions                                 #
#############################################################

Draw.Register(draw, event, bevent)

