#!BPY

"""
Name: 'BMG spinner!'
Blender: 234
Group: 'Animation'
Tooltip: 'Make BMG made gears spin correctly'
"""

#############################################################
#                                                           #
# Blender Mechanical Gears SPINNER!                         #
#                                                           #
# (C) November 2004 Stefano <S68> Selleri                   #
# Released under the Blender Artistic Licence (BAL)         #
# See www.blender.org                                       #
#                                                           #
#############################################################
# History                                                   #
# V: 0.0.0 - 13-11-04 - The script starts to take shape, an #
#                       history is now deserved :)          #
# V: 0.0.2 - 26-11-04 - Well, no changes, but let's keep    #
#                       matching version numbers...         #
#############################################################

import Blender
from Blender import *
from math import *

#############################################################
# GLOBALS                                                   #
#############################################################
VERSION         ='0.0.0'

#############################################################
# INTERFACE EVENTS                                          #
#############################################################
EVENTHASH = {
    'NOEVENT': 1,
    'REDRAW': 2,
    'EXIT': 3,
    'DO': 4,
    'SAVE': 5,
    'LOAD': 6,

    'ADDGEAR' : 200,
    'ADDLINK' : 300,
    
    'UPLINK' : 400,
    'DWLINK' : 500,

    'DLG_OK' : 600
    }

#############################################################
# GLOBALS                                                   #
#############################################################
STATEHASH = {
    'GEARS' : Draw.Create(1), 'GEARS_EV' : 100,
    'LINKS' : Draw.Create(0), 'LINKS_EV' : 101,
    }

GEARNAMES  = []
GEARTEETH  = []
GEAREVENTS = []

LINKGEARG  = []
LINKGEARP  = []
LINKTYPE   = []
LINKEVENTS = []
LINKUP     = []
LINKDW     = []

####################################################################
#CREATES THE SCRIPTLINK!
####################################################################
def DoScriptLink():
    print "OK"

    try:
	tb = Text.Get('DriverSL')
    except:
	tb = Text.New('DriverSL')

    tb.clear()

    tb.write("#########################################################\n")
    tb.write("# This Test buffer is automatically generated by\n")
    tb.write("# BMGs "+VERSION+"\n");
    tb.write("#########################################################\n")
    tb.write("# Add it as a scene-wise script link\n\n")

    tb.write("import Blender\n")
    tb.write("from   Blender import *\n")
    tb.write("fn    =  Get('curframe')\n\n")

    for i in range(len(GEARNAMES)):
	tb.write(GEARNAMES[i].val+" = Object.Get('"+GEARNAMES[i].val+"')\n")

    tb.write("\n")

    for i in range(len(LINKGEARG)):
	ig   = LINKGEARG[i].val
	ip   = LINKGEARP[i].val
	Type = LINKTYPE[i].val

	if (Type == 0):
	    Ratio = 1.0
	elif (Type == 1):
	    Ratio = - 1.0 * GEARTEETH[ig].val / GEARTEETH[ip].val
        else:
            Ratio = 1.0 * GEARTEETH[ig].val / GEARTEETH[ip].val

	tb.write("euA = "+GEARNAMES[ig].val+".getEuler()\n")
	tb.write("euB = "+GEARNAMES[ip].val+".getEuler()\n")
	tb.write("euB[2] = "+str(Ratio)+" * euA[2]\n")
	tb.write(GEARNAMES[ip].val+".setEuler(euB)\n\n")
	

    
        
    
#############################################################
# MAIN WINDOW                                               #
#############################################################
def MainWindow():        
    global EVENTHASH,STATEHASH
    global TOOTHHASH
    
    #############################################################
    # Backgrounds                                               #
    #############################################################

    BGL.glClearColor(0.5, 0.5, 0.5, 0.0)
    BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
    BGL.glColor3f(0, 0, 0) 			# Black
    BGL.glRectf(2, 2, 482, 520)
    BGL.glColor3f(0.48, 0.4, 0.57) 		# Light Purple
    BGL.glRectf(4, 479, 480, 510)
    BGL.glRectf(4, 34, 480, 450)
    BGL.glColor3f(0.3, 0.27, 0.35) 		# Dark purple
    BGL.glRectf(4, 451,480, 478)
    BGL.glRectf(4, 4, 480, 33)

    #############################################################
    # Common Header                                             #
    #############################################################

    BGL.glColor3f(0.9, 0.8, 1)
    BGL.glRasterPos2d(10, 500)
    Draw.Text("BMG SPINNER! v. "+VERSION)
    BGL.glRasterPos2d(10, 484)
    Draw.Text("(C) Nov 2004 Stefano Selleri <a.k.a. S68>")
#    Draw.Button("Save", EVENTHASH['SAVE'], 267, 482, 70, 24,
#        	"Save currents settings to a file")
#    Draw.Button("Load", EVENTHASH['LOAD'], 337, 482, 70, 24,
#                "Loads Previously saved Settings form a file")
    Draw.Button("Exit", EVENTHASH['EXIT'], 407, 482, 70, 24)
 
    STATEHASH['GEARS'] = Draw.Toggle(
        "Gears",STATEHASH['GEARS_EV'],10,454,65,20,STATEHASH['GEARS'].val,
        "Gear Database")
    STATEHASH['LINKS'] = Draw.Toggle(
        "Links",STATEHASH['LINKS_EV'],75,454,65,20,STATEHASH['LINKS'].val,
        "Links Database")
        
    Draw.Button("GENERATE", EVENTHASH['DO'], 7, 7, 470, 24)
    BGL.glColor3f(0.9, 0.9, 0.9)

    #############################################################
    # Screen                                                    #
    #############################################################
    if (STATEHASH['GEARS'].val==1):
        BGL.glRasterPos2d(10, 425)
        Draw.Text("Gears Database:")

        off = 0
        for i in range(len(GEARNAMES)):
            if (i>15):
                off = 1

            GEARNAMES[i] = Draw.String("Ob: ", EVENTHASH['NOEVENT'],
                                        10+off*220, 400-(i-off*16)*20,
                                        100, 18, GEARNAMES[i].val,
                                        8 , "Name of the Gear Object")
            GEARTEETH[i] = Draw.Number("Te: ", EVENTHASH['NOEVENT'],
                                        110+off*220, 400-(i-off*16)*20,
                                        80, 18, GEARTEETH[i].val,
                                        1 , 1000, "Teeth in the Gear")
            Draw.Button("Del", GEAREVENTS[i], 190 + off*220,
                        400-(i-off*16)*20, 30, 18, "Remove this Gear data")

        i = len(GEARNAMES)
        if (i>15):
            off = 1

        if (len(GEARNAMES)<32):    
            Draw.Button("Add", EVENTHASH['ADDGEAR'],
                        190+off*220, 400-(i-off*16)*20, 30, 18,
                        "Add a new gear")
        
    else:
        BGL.glRasterPos2d(10, 425)
        Draw.Text("Links Database:")
        nameG = "Driver %t"
        nameP = "Driven %t"
        TypeL = "Type %t|Fixed %x0|Mesh %x1|InvMesh %x2"
        for i in range(len(GEARNAMES)):
            nameG = nameG + "|"+GEARNAMES[i].val+" %x"+str(i)
            nameP = nameP + "|"+GEARNAMES[i].val+" %x"+str(i)
            
        off = 0
        for i in range(len(LINKGEARG)):
            if (i>15):
                off = 1

            LINKGEARG[i] = Draw.Menu(nameG, EVENTHASH['NOEVENT'],
                                     10+off*240, 400-(i-off*16)*20,
                                     70, 18, LINKGEARG[i].val,
                                     "Gear Driving this pair")
            LINKGEARP[i] = Draw.Menu(nameP, EVENTHASH['NOEVENT'],
                                     80+off*240, 400-(i-off*16)*20,
                                     70, 18, LINKGEARP[i].val,
                                     "Driven Gear in this pair")
            LINKTYPE[i] = Draw.Menu(TypeL, EVENTHASH['NOEVENT'],
                                     150+off*240, 400-(i-off*16)*20,
                                     35, 18, LINKTYPE[i].val,
                                     "Driven Gear in this pair")
            Draw.Button("", LINKUP[i], 185 + off*240,
                        400-(i-off*16)*20+9, 10, 9, "Move UP")
            Draw.Button("", LINKDW[i], 185 + off*240,
                        400-(i-off*16)*20, 10, 9, "Move DOWN")

            Draw.Button("Del", LINKEVENTS[i], 195 + off*240,
                        400-(i-off*16)*20, 30, 18, "Remove this link")

        i = len(LINKGEARG)
        if (i>15):
            off = 1

        if (len(LINKGEARG)<32):    
            Draw.Button("Add", EVENTHASH['ADDLINK'],
                        195+off*240, 400-(i-off*16)*20, 30, 18, "Add new link")

#############################################################
# Graphics                                                  #
#############################################################
def draw():
    MainWindow()
    
#############################################################
# Event Handler                                             #
#############################################################
def event(evt, val): 
	if (evt== Draw.QKEY and not val):
		Draw.Exit()

def bevent(evt):
    global EVENTHASH, STATEHASH, GEAREVENTS, LINKEVENTS
    print evt
    if   (evt == EVENTHASH['EXIT']):
        print 'EXIT'
        Draw.Exit()
    elif (evt == EVENTHASH['DO']):
        DoScriptLink()
        Draw.Redraw()
	
    elif (evt == EVENTHASH['SAVE']):
        print 'SAVE'
        Draw.Redraw()
	Window.FileSelector (Save, 'SAVE FILE:')        
    elif (evt == EVENTHASH['LOAD']):
        print 'LOAD'
        Draw.Redraw()
	Window.FileSelector (Load, 'LOAD FILE:')        
        
    elif (evt == STATEHASH['GEARS_EV']):
        print 'GEARS_EV'
	STATEHASH['GEARS'].val     = 1
	STATEHASH['LINKS'].val     = 0
	Draw.Redraw()
    elif (evt == STATEHASH['LINKS_EV']):
        print 'LINKS_EV'
        STATEHASH['GEARS'].val     = 0
	STATEHASH['LINKS'].val     = 1
	Draw.Redraw()
        
    elif (evt == EVENTHASH['ADDGEAR']):
        print 'OK'
        GEARNAMES.append(Draw.Create('Gear'))
        GEARTEETH.append(Draw.Create(20))
        GEAREVENTS.append(EVENTHASH['ADDGEAR']+len(GEARTEETH))
        Draw.Redraw()
        
    elif (evt > EVENTHASH['ADDGEAR'] and evt < EVENTHASH['ADDGEAR']+80):
        for i in range(len(GEARNAMES)):
            if (evt == GEAREVENTS[i]):
                GEARNAMES.pop(i)
                GEARTEETH.pop(i)
                break
        GEAREVENTS = []
        for i in range (len(GEARNAMES)):
            GEAREVENTS.append(EVENTHASH['ADDGEAR'] + i + 1)
        Draw.Redraw()

    elif (evt == EVENTHASH['ADDLINK']):
        print 'OK'
        LINKGEARG.append(Draw.Create(0))
        LINKGEARP.append(Draw.Create(0))
        LINKTYPE.append(Draw.Create(0))
        LINKEVENTS.append(EVENTHASH['ADDLINK']+len(LINKGEARG))
        LINKUP.append(EVENTHASH['UPLINK']+len(LINKGEARG))
        LINKDW.append(EVENTHASH['DWLINK']+len(LINKGEARG))
        Draw.Redraw()
        
    elif (evt > EVENTHASH['ADDLINK'] and evt < EVENTHASH['ADDLINK']+80):
        for i in range(len(LINKGEARG)):
            if (evt == LINKEVENTS[i]):
                LINKGEARG.pop(i)
                LINKGEARP.pop(i)
                LINKTYPE.pop(i)
                break
        LINKEVENTS = []
        for i in range (len(LINKGEARG)):
            LINKEVENTS.append(EVENTHASH['ADDLINK'] + i + 1)
            LINKUP.append(EVENTHASH['UPLINK']+i+1)
            LINKDW.append(EVENTHASH['DWLINK']+i+1)
        Draw.Redraw()

    elif (evt > EVENTHASH['UPLINK'] and evt < EVENTHASH['UPLINK']+80):
        for i in range(len(LINKGEARG)):
            if (evt == LINKUP[i]):
                idx = i
                break
        if (idx>0):
            Aux               = LINKGEARG[idx]
            LINKGEARG[idx]    = LINKGEARG[idx-1]
            LINKGEARG[idx-1 ] = Aux
            Aux               = LINKGEARP[idx]
            LINKGEARP[idx]    = LINKGEARP[idx-1]
            LINKGEARP[idx-1 ] = Aux
            Aux               = LINKTYPE[idx]
            LINKTYPE[idx]     = LINKTYPE[idx-1]
            LINKTYPE[idx-1 ]  = Aux
        Draw.Redraw()

    elif (evt > EVENTHASH['DWLINK'] and evt < EVENTHASH['DWLINK']+80):
        for i in range(len(LINKGEARG)):
            if (evt == LINKDW[i]):
                idx = i
                break
        if (idx<len(LINKGEARG)-1):
            Aux               = LINKGEARG[idx]
            LINKGEARG[idx]    = LINKGEARG[idx+1]
            LINKGEARG[idx+1 ] = Aux
            Aux               = LINKGEARP[idx]
            LINKGEARP[idx]    = LINKGEARP[idx+1]
            LINKGEARP[idx+1 ] = Aux
            Aux               = LINKTYPE[idx]
            LINKTYPE[idx]     = LINKTYPE[idx+1]
            LINKTYPE[idx+1 ]  = Aux
        Draw.Redraw()

    else:
        print 'Che ci faccio qui?'
#############################################################
# Registering all functions                                 #
#############################################################

Draw.Register(draw, event, bevent)
