#!BPY

"""
Name: 'Blender Rope Works GUI'
Blender: 236
Group: 'Animation'
Tooltip: 'GUI for real BRWs.py script'
"""

#############################################################
#                                                           #
# Blender Rope Works GUI1                                   #
#                                                           #
# (C) March 2005 Stefano <S68> Selleri                      #
# Released under the Blender Artistic Licence (BAL)         #
# See www.blender.org                                       #
#                                                           #
#############################################################
# History                                                   #
# V: 0.0.0 - 12-03-05 - The real script, BRWs.py starts to  #
#                       work, so now we need a GUI          #
#    0.0.2 - 14-03-05 - Added Help, sincronized version n.  #
#############################################################

import Blender
from Blender import *
import re

#############################################################
# GLOBALS                                                   #
#############################################################
VERSION         ='0.0.2'

#############################################################
# INTERFACE EVENTS                                          #
#############################################################
EVENTHASH = {
    'NOEVENT': 1,
    'REDRAW': 2,
    'EXIT': 3,
    'DO': 4,
    'REFRESH': 5,
    'STORE': 6,
    
    'ACTION_T': 100,
    'ACTION_H': 200
    }

#############################################################
# GLOBALS                                                   #
#############################################################
STATEHASH = {}
GUIHASH   = {}
HELP      = Draw.Create(0)

EndA = {}
EndB = {}
P    = {}
C    = {}
Rope = {}

dis  = {}
rad  = {}

#############################################################
# REFRESH                                                   #
#############################################################
def DoRefresh():

    global EndA,EndB,P,C,Rope,dis,rad,STATEHASH

    ##### PERSISTENT DATA
    dict = Registry.GetKey('BRW')
    try:
        dis = dict['dis']
    except:
        dis = {}
    
    try:
        rad = dict['rad']
    except:
        rad = {}

    EndA = {}
    EndB = {}
    P    = {}
    C    = {}
    Rope = {}

    OL = Object.Get();

    ########## Extracts Ends, Points, Centers, Ropes
    MaxState = 0
    for o in OL:
        try:
            i = int(re.search(r"^EndA\.(\d*)",o.name).group(1))
            if (i>MaxState):
                MaxState = i
            EndA[i] = o
        except:
            pass

        try:
            i = int(re.search(r"^EndB\.(\d*)",o.name).group(1))
            if (i>MaxState):
                MaxState = i
            EndB[i] = o
        except:
            pass

        try:
            i = int(re.search(r"^P\.(\d*)\.(\d*)",o.name).group(1))
            if (i>MaxState):
                MaxState = i
            j = int(re.search(r"^P\.(\d*)\.(\d*)",o.name).group(2))
            try:
                P[i][j] = o
            except:
                P[i] = {}
                P[i][j] = o
        except:
            pass

        try:
            i = int(re.search(r"^C\.(\d*)\.(\d*)",o.name).group(1))
            if (i>MaxState):
                MaxState = i
            j = int(re.search(r"^C\.(\d*)\.(\d*)",o.name).group(2))
            try:
                C[i][j] = o
            except:
                C[i] = {}
                C[i][j] = o   
        except:
            pass

        try:
            i = int(re.search(r"^Rope\.(\d*)",o.name).group(1))
            if (i>MaxState):
                MaxState = i
            Rope[i] = o
        except:
            pass

    ##### TABS 
    for j in range(1,MaxState+1):
        if j == 1:
            STATEHASH[j] = Draw.Create(1)
        else:
            STATEHASH[j] = Draw.Create(0)

    ##### SLIDERS
    try:
        for i in C.keys():
            try:
                for j in C[i].keys():
                    try:
                        aux = dis[i][j]
                    except:
                        try:
                            aux = dis[i]
                            dis[i][j] = Draw.Create(0.25)
                        except:
                            dis[i] = {}
                            dis[i][j] = Draw.Create(0.25)
                    try:
                        aux = rad[i][j]
                    except:
                        try:
                            aux = rad[i]
                            rad[i][j] = Draw.Create(0.1)
                        except:
                            rad[i] = {}
                            rad[i][j] = Draw.Create(0.1)
            except:
                pass
    except:
        pass

#############################################################
# STORE                                                     #
#############################################################
def DoStore():

    global dis,rad

    ##### PERSISTENT DATA
    dict = {}
    dict['dis'] = dis
    dict['rad'] = rad
    Registry.SetKey('BRW',dict)
    
#############################################################
# Graphics                                                  #
#############################################################
def draw():
    global EVENTHASH,STATEHASH
    global GUIHASH,HELP
    global EndA,EndB,P,C,Rope,dis,rad
    
    #############################################################
    # Backgrounds                                               #
    #############################################################

    BGL.glClearColor(0.5, 0.5, 0.5, 0.0)
    BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
    BGL.glColor3f(0, 0, 0) 			# Black
    BGL.glRectf(2, 2, 482, 520)
    BGL.glColor3f(0.48, 0.4, 0.57) 		# Light Purple
    BGL.glRectf(4, 479, 480, 510)
    BGL.glRectf(4, 34, 480, 450)
    BGL.glColor3f(0.3, 0.27, 0.35) 		# Dark purple
    BGL.glRectf(4, 451,480, 478)
    BGL.glRectf(4, 4, 480, 33)

    #############################################################
    # Common Header                                             #
    #############################################################

    BGL.glColor3f(0.9, 0.8, 1)
    BGL.glRasterPos2d(10, 500)
    Draw.Text("Blender Rope Works GUI v. "+VERSION)
    BGL.glRasterPos2d(10, 484)
    Draw.Text("(C) March 2005 Stefano Selleri <a.k.a. S68>")
    Draw.Button("Exit", EVENTHASH['EXIT'], 407, 482, 70, 24)

    ## STATE
    for i in STATEHASH.keys():
        STATEHASH[i] = Draw.Toggle(str(i),EVENTHASH['ACTION_T']+i,
                                   10+(i-1)*20,455,18,18,
                                   STATEHASH[i].val,
                                   "Rope "+str(i)+"settings")
    
    HELP = Draw.Toggle('HELP',EVENTHASH['ACTION_H'],
                       438,455,38,18,HELP.val,
                       "Help")

    Draw.Button("REFRESH", EVENTHASH['REFRESH'], 7, 7, 235, 24)
    Draw.Button("STORE", EVENTHASH['STORE'], 242, 7, 235, 24)
    BGL.glColor3f(0.9, 0.9, 0.9)

    #############################################################
    # Screen                                                    #
    #############################################################
    idx = -1
    for i in STATEHASH.keys():
        if (STATEHASH[i].val == 1):
            idx = i

    if (idx>=0):
        BGL.glRasterPos2d(10, 425)
        Draw.Text("Items of Rope "+str(idx)+":")
        ##### ENDA
        try:
            BGL.glRasterPos2d(20, 405)
            Draw.Text(EndA[idx].name)
            EA = 1
        except:
            EA = 0

        ##### POINTS
        Pnum = 0
        Maxj = 0
        try:
            for i in P[idx].keys():
                BGL.glRasterPos2d(20, 405-i*20)
                Draw.Text(P[idx][i].name)
                Pnum = Pnum + 1
                if (i>Maxj):
                    Maxj = i
        except:
            pass

        ##### CENTERS
        Cnum = 0
        try:
            for i in C[idx].keys():
                BGL.glRasterPos2d(80, 405-i*20)
                Draw.Text(C[idx][i].name)
                Cnum = Cnum + 1
                if (i>Maxj):
                    Maxj = i
        except:
            pass
    
        ##### DISTANCES
        try:
            for i in dis[idx].keys():
                dis[idx][i] = Draw.Number("d: ", EVENTHASH['NOEVENT'],
                                          140, 400-i*20,
                                          80, 18, dis[idx][i].val,
                                          0.0 , 100.0, "P -> C Distance")

        except:
            pass

        ##### RADII
        try:
            for j in rad[idx].keys():
                rad[idx][j] = Draw.Number("r: ", EVENTHASH['NOEVENT'],
                                          220, 400-j*20,
                                          80, 18, rad[idx][j].val,
                                          0.0 , 100.0, "Pulley Radius")

        except:
            pass

        ##### ENDB
        try:
            BGL.glRasterPos2d(20, 385-Maxj*20)
            Draw.Text(EndB[idx].name)
            EB = 1
        except:
            EB = 0

        ##### WARNINGS
        if (EA == 0 or Pnum != Cnum or EB == 0):
            BGL.glRasterPos2d(20, 365-Maxj*20)
            BGL.glColor3f(0.6, 0.0, 0.0)
            BGL.glRectf(10, 360-max(Pnum,Cnum)*20, 470, 380-max(Pnum,Cnum)*20)
            BGL.glColor3f(0.9, 0.9, 0.9)
            Draw.Text("WARNING - Missing EndA, EndB or non-matching P and C")
            
        try:
            BC = Rope[idx].getData()
            i = 0;
            try:
                while (BC.getControlPoint(0,i)):
                    i = i+1
            except:
                pass
        except:
            i = -1
    
        if ((i-2)/3-Cnum != 0):
            BGL.glColor3f(0.6, 0.0, 0.0)
            BGL.glRectf(10, 335-max(Pnum,Cnum)*20, 470, 355-max(Pnum,Cnum)*20)
            BGL.glColor3f(0.9, 0.9, 0.9)        
            BGL.glRasterPos2d(20, 340-Maxj*20)
            Draw.Text("WARNING - Missing Rope or Control points are not 2 + 3*Number of C points")
    else:
        ##### HELP!
        BGL.glRasterPos2d(10, 425)
        Draw.Text("HELP:")

        BGL.glRasterPos2d(10, 405)
        Draw.Text('  BRW is a rather picky script needing many stuff to be properly set-up. It Takes')
        BGL.glRasterPos2d(10, 385)
        Draw.Text('an existing bezier and wind it around a given set of pulleys. It can handle as many')
        BGL.glRasterPos2d(10, 365)
        Draw.Text('ropes as you wish. Let\'s define \'i\' an index, from 1 to N defining a given rope.')

        BGL.glRasterPos2d(10, 340)
        Draw.Text('  Now, you need to have two empties, EndA.i and EndB.i for each rope. Please note')
        BGL.glRasterPos2d(10, 320)
        Draw.Text('the index of rope \'i\'. The rope will stretch from EndA.i to EndB.i. Then you need ')
        BGL.glRasterPos2d(10, 300)
        Draw.Text('to define Pulleys. Pulleys are identified via index \'j\', in the 1 to M(i) range, M')
        BGL.glRasterPos2d(10, 280)
        Draw.Text('depending on Rope. Pulleys have a center which can be fixed or can be free to')
        BGL.glRasterPos2d(10, 260)
        Draw.Text('rotate around a fixed point. You need a pair of Empties for each pulley, named P.i.j')
        BGL.glRasterPos2d(10, 240)
        Draw.Text('and C.i.j. P.i.j is the fixed point, C.i.j is the center of the pulley. C.i.j is automatically')
        BGL.glRasterPos2d(10, 220)
        Draw.Text('placed by the script at a specified distance \'d\' from P.i.j This latter is either truly')
        BGL.glRasterPos2d(10, 200)
        Draw.Text('fixed or can be animated via Ipo. All the rest will follow. The script will also take')
        BGL.glRasterPos2d(10, 180)
        Draw.Text('care of winding the Rope.i bezier around C.i.j with a winding radius \'r\'.') 

        BGL.glRasterPos2d(10, 155)
        Draw.Text('  To allow all this winding Rope.i object must have a number of Control Points equal')
        BGL.glRasterPos2d(10, 135)
        Draw.Text('to 2 + 3*M(i). Furthermore, while EndA.i, EndB.i and P.i.j empties can be animated')
        BGL.glRasterPos2d(10, 115)
        Draw.Text('or can be parented to any object, animated or not, C.i.j empties and Rope.i must be')
        BGL.glRasterPos2d(10, 95)
        Draw.Text('unparented... they are anyway placed correctly by script.') 

        BGL.glRasterPos2d(10, 70)
        Draw.Text('Once you are done press \'STORE\' or \'Exit\'. Then you must add BRWs script, with')
        BGL.glRasterPos2d(10, 50)
        Draw.Text('*matching version number* as a scene scriptlink on FrameChanged event.') 
        
#############################################################
# Event Handler                                             #
#############################################################
def event(evt, val): 
	if (evt== Draw.QKEY and not val):
		Draw.Exit()

def bevent(evt):
    global EVENTHASH, STATEHASH, HELP
    
    if   (evt == EVENTHASH['EXIT']):
        # EXIT
        DoStore()
        Draw.Exit()
        # 2 = NOOP
    elif (evt == EVENTHASH['DO']):
        DoTheTooth()
	Draw.Redraw()

    elif (evt == EVENTHASH['REFRESH']):
        DoRefresh()
	Draw.Redraw()
    elif (evt == EVENTHASH['STORE']):
        DoStore()
	Draw.Redraw()
        
    elif (evt >= EVENTHASH['ACTION_T'] and evt <= EVENTHASH['ACTION_T']+50):
        j = evt - EVENTHASH['ACTION_T']
        for i in STATEHASH.keys():
            STATEHASH[i].val = 0
        STATEHASH[j].val = 1
        HELP.val = 0
	Draw.Redraw()

    elif (evt == EVENTHASH['ACTION_H']):
        for i in STATEHASH.keys():
            STATEHASH[i].val = 0
        HELP.val = 1
	Draw.Redraw()
        
#############################################################
# Registering all functions                                 #
#############################################################
DoRefresh()
Draw.Register(draw, event, bevent)
